'use strict';

var React = require('react');
var PureRenderMixin = require('react/lib/ReactComponentWithPureRenderMixin');

var _require = require('react-motion');

var Spring = _require.Spring;

var objectAssign = require('object-assign');

var styles = {
  root: {
    overflowX: 'hidden'
  },
  container: {
    display: 'flex'
  },
  slide: {
    width: '100%',
    flexShrink: 0,
    overflow: 'auto'
  }
};

var SwipeableViews = React.createClass({
  displayName: 'SwipeableViews',

  propTypes: {
    children: React.PropTypes.node,
    index: React.PropTypes.number,
    onChangeIndex: React.PropTypes.func,
    style: React.PropTypes.object
  },
  mixins: [PureRenderMixin],
  getDefaultProps: function getDefaultProps() {
    return {
      index: 0
    };
  },
  getInitialState: function getInitialState() {
    return {
      index: this.props.index,
      isDragging: false,
      isFirstRender: true
    };
  },
  componentDidMount: function componentDidMount() {
    this.setState({
      isFirstRender: false
    });
  },
  componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
    var index = nextProps.index;

    if (typeof index === 'number' && index !== this.props.index) {
      this.setState({
        index: index
      });
    }
  },
  handleTouchStart: function handleTouchStart(event) {
    var touch = event.touches[0];

    this.startWidth = React.findDOMNode(this).getBoundingClientRect().width;
    this.startIndex = this.state.index;
    this.startX = touch.pageX;
    this.lastX = touch.pageX;
    this.deltaX = 0;
    this.startY = touch.pageY;
    this.isScroll = undefined;
  },
  handleTouchMove: function handleTouchMove(event) {
    var touch = event.touches[0];

    // This is a one time test
    if (this.isScroll === undefined) {
      this.isScroll = Math.abs(this.startY - touch.pageY) > Math.abs(this.startX - touch.pageX);
    }

    if (this.isScroll) {
      return;
    }

    // Prevent native scrolling
    event.preventDefault();

    this.deltaX = touch.pageX - this.lastX;
    this.lastX = touch.pageX;

    var indexMax = React.Children.count(this.props.children) - 1;

    var index = this.startIndex + (this.startX - touch.pageX) / this.startWidth;

    if (index < 0) {
      index = 0;
    } else if (index > indexMax) {
      index = indexMax;
    }

    this.setState({
      isDragging: true,
      index: index
    });
  },
  handleTouchEnd: function handleTouchEnd() {
    if (this.isScroll) {
      return;
    }

    var indexNew = undefined;

    // Quick movement
    if (Math.abs(this.deltaX) > 10) {
      if (this.deltaX > 0) {
        indexNew = Math.floor(this.state.index);
      } else {
        indexNew = Math.ceil(this.state.index);
      }
    } else {
      // Some hysteresis with startIndex
      if (Math.abs(this.startIndex - this.state.index) > 0.6) {
        indexNew = Math.round(this.state.index);
      } else {
        indexNew = this.startIndex;
      }
    }

    this.setState({
      index: indexNew,
      isDragging: false
    });

    if (this.props.onChangeIndex && indexNew !== this.startIndex) {
      this.props.onChangeIndex(indexNew);
    }
  },
  renderContainer: function renderContainer(translate) {
    var _props = this.props;
    var children = _props.children;
    var style = _props.style;
    var isFirstRender = this.state.isFirstRender;

    var childrenToRender = undefined;

    if (isFirstRender) {
      childrenToRender = children[0];
    } else {
      childrenToRender = React.Children.map(children, function (element) {
        return React.createElement(
          'div',
          { style: styles.slide },
          element
        );
      });
    }

    return React.createElement(
      'div',
      { style: objectAssign({
          WebkitTransform: 'translate3d(' + translate + 'px, 0, 0)',
          transform: 'translate3d(-' + translate + '%, 0, 0)'
        }, styles.container, style) },
      childrenToRender
    );
  },
  render: function render() {
    var _this = this;

    var _state = this.state;
    var index = _state.index;
    var isDragging = _state.isDragging;

    var endValue = {
      val: index * 100,
      config: isDragging ? [3000, 50] : [300, 30]
    };

    return React.createElement(
      'div',
      { style: styles.root,
        onTouchStart: this.handleTouchStart, onTouchMove: this.handleTouchMove, onTouchEnd: this.handleTouchEnd },
      React.createElement(
        Spring,
        { endValue: endValue },
        function (interpolated) {
          return _this.renderContainer(interpolated.val);
        }
      )
    );
  }
});

module.exports = SwipeableViews;